import sys
import os

sys.path.insert(0, os.path.curdir)

from meo.crack import Crack
import meo

# pdf = Crack.pdf("./examples/data/cannot_edit.pdf")
# pdf.remove_pdf_password("./output.pdf")
# pdf = Crack.pdf("./examples/data/cannot_open.pdf")
# pdf.remove_pdf_password("./output.pdf")
wait = Crack.zip("./test/flag.zip")
res = wait.in_multiprocess_by_seed(meo.crack.utils.PWD_SEED_COMPLEX, progressbar=0, gc_interval=100)
print(res)


'''
    function may be used in flask app.
'''
def allow_cors(flask_app):
    """
        allow cors after requet.
    """
    @flask_app.after_request
    def cors(environ):
        environ.headers['Access-Control-Allow-Origin'] = '*'
        environ.headers['Access-Control-Allow-Method'] = '*'
        environ.headers["Access-Control-Allow-Credentials"] = 'true'
        environ.headers['Access-Control-Allow-Headers'] = 'Content-Type, Depth, User-Agent, " \
            "X-File-Size, X-Requested-With, If-Modified-Since, X-File-Name, Cache-Control'
        return environ



class UserAgent:

    """UA in different platform"""

    FIREFOX = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:102.0) Gecko/20100101 Firefox/102.0"

    SAFARI_IPHONE = "Mozilla/5.0 (iPhone; CPU iPhone OS 15_5 like Mac OS X) " \
                    "AppleWebKit/605.1.15 (KHTML, like Gecko) " \
                    "Version/15.4 Mobile/15E148 Safari/604.1"

    SAFARI_IPAD = "Mozilla/5.0 (iPad; CPU OS 15_5 like Mac OS X) " \
                    "AppleWebKit/605.1.15 (KHTML, like Gecko) " \
                    "Version/15.4 Mobile/15E148 Safari/604.1"

    SAFARI_IPOD = "Mozilla/5.0 (iPod touch; CPU iPhone 15_5 like Mac OS X) " \
                    "AppleWebKit/605.1.15 (KHTML, like Gecko) " \
                    "Version/15.4 Mobile/15E148 Safari/604.1"


if __name__ == '__main__':
    print(UserAgent.FIREFOX)


'''
    Display input and output
'''

SYMBOL_PLAIN          = "\33[0m"
SYMBOL_CURSOR_HIDDEN  = "\33[?25l"
SYMBOL_CURSOR_DISPLAY = "\33[?25h"
SYMBOL_BLANKING       = "\33[8m"

FORMAT_FONT_BLACK    = "\33[30;1m{}\33[0m"
FORMAT_FONT_RED      = "\33[31;1m{}\33[0m"
FORMAT_FONT_GREEN    = "\33[32;1m{}\33[0m"
FORMAT_FONT_YELLOW   = "\33[33;1m{}\33[0m"
FORMAT_FONT_BLUE     = "\33[34;1m{}\33[0m"
FORMAT_FONT_PRUPLE   = "\33[35;1m{}\33[0m"
FORMAT_FONT_SKY_BLUE = "\33[36;1m{}\33[0m"
FORMAT_FONT_WHITE    = "\33[37;1m{}\33[0m"

FORMAT_BG_BLACK    = "\33[40;1m{}\33[0m"
FORMAT_BG_RED      = "\33[41;1m{}\33[0m"
FORMAT_BG_GREEN    = "\33[42;1m{}\33[0m"
FORMAT_BG_YELLOW   = "\33[43;1m{}\33[0m"
FORMAT_BG_BLUE     = "\33[44;1m{}\33[0m"
FORMAT_BG_PRUPLE   = "\33[45;1m{}\33[0m"
FORMAT_BG_SKY_BLUE = "\33[46;1m{}\33[0m"
FORMAT_BG_WHITE    = "\33[47;1m{}\33[0m"

FORMAT_UNDERLINE       = "\33[4m{}\33[0m"
FORMAT_TWINKLE         = "\33[5m{}\33[0m"
FORMAT_REVERSE_DISPLAY = "\33[7m{}\33[0m"
FORMAT_BLANKING        = "\33[8m{}\33[0m"

FORMAT_CURSOR_MOVE_UP    = "\33[{}A"
FORMAT_CURSOR_MOVE_DOWN  = "\33[{}B"
FORMAT_CURSOR_MOVE_RIGHT = "\33[{}C"
FORMAT_CURSOR_MOVE_LEFT  = "\33[{}D"

def red_font(text, end='\n'):
    '''print ```text``` red font'''
    print(FORMAT_FONT_RED.format(text), end=end)

def green_font(text, end="\n"):
    '''print ```text``` green font'''
    print(FORMAT_FONT_GREEN.format(text), end=end)

def blue_font(text, end="\n"):
    '''print ```text``` blue font'''
    print(FORMAT_FONT_BLUE.format(text), end=end)

def sformat(text, *styles):
    '''format string```text```'''
    for _s in styles:
        text = _s.format(text)
    return text

def clear_screen():
    '''clear screen'''
    print("\33[2J")

def hidden_input(__prompt: object):
    """ hidden keyboard input """
    text = input("{}{}{}".format(__prompt, SYMBOL_CURSOR_HIDDEN, SYMBOL_BLANKING))
    print("{}{}".format(SYMBOL_PLAIN, SYMBOL_CURSOR_DISPLAY), end='')
    return text

if __name__ == "__main__":

    # _text = sformat("hello world", FORMAT_BG_BLUE, FORMAT_FONT_YELLOW, FORMAT_UNDERLINE)
    # print(_text)
    # print(FORMAT_CURSOR_MOVE_UP.format(20))
    # clear_screen()
    # __psw = hidden_input("psw: ")
    # print(__psw)
    ...



from pikepdf import Pdf, PasswordError
from .cracker import Cracker

class PdfCracker(Cracker):
    def remove_pdf_password(self, output_path: str, pwd=b''):
        with Pdf.open(self.path, password=pwd) as f:
            f.save(output_path)
            
    def test(self, pwd=b''):
        try:
            with Pdf.open(self.path, password=pwd) as f:
                return True
        except PasswordError:
            return False

    def step_once(self, pwd: bytes) -> bytes | None:
        if self.test(pwd):
            return pwd
        return None
    
    def step_some(self, *pwds: bytes) -> bytes | None:
        for pwd in pwds:
            if self.test(pwd):
                return pwd
        return None



"""
    awesome functions
"""
import os

def script_path(__file__):
    """return script real path in disk."""
    return os.path.split(os.path.realpath(__file__))[0]

class SimplePath:
    def __init__(self, path: str) -> None:
        self.path = path

    def join(self, *paths):
        path = os.path.join(self.path, *paths)
        return SimplePath(path)

def ScriptPath(__file__):
    return SimplePath(script_path(__file__))



from pyzipper import ZipFile
from .cracker import Cracker
from ..io import encode

class ZipCracker(Cracker):
    
    def test(self, zfile: ZipFile, pwd: bytes, mem: str):
        try:
            assert isinstance(pwd, bytes)
        except AssertionError:
            pwd = encode(pwd)
        try:
            with zfile.open(mem, pwd=pwd) as f:
                if f.seek(1):
                    return True
        except ValueError as e:
            raise e
        except:
            return False

    def step_once(self, pwd: bytes) -> bytes | None:
        zfile = ZipFile(self.path)
        mem = zfile.filelist[0].filename
        if self.test(zfile, pwd, mem):
            return pwd
        del pwd
        return None
    
    def step_some(self, *pwds: bytes) -> bytes | None:
        zfile = ZipFile(self.path)
        mem = zfile.filelist[0].filename
        for pwd in pwds:
            if self.test(zfile, pwd, mem):
                return pwd
        del pwds
        return None



import time

def timeit(func):
    def wapper(*args, **kwargs):
        st = time.time()
        res = func(*args, **kwargs)
        ed = time.time()
        return ed - st, res
    return wapper

if __name__ == '__main__':
    @timeit
    def test(k):
        time.sleep(5)
        print(k)
        return k ** 2
    print(test(3))


""" IO of formatted file and object """
import json5
import json
import os

def checkout_dir(dir_path):
    """Create if the folder does not exist"""
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)

def checkout_file_path(path):
    fdir, _ = os.path.split(path)
    checkout_dir(fdir)

def to_json5(obj, path: str, encoding='utf8', ensure_ascii=False, indent=None, auto_mkdirs=True):
    if auto_mkdirs:
        checkout_file_path(path)
    with open(path, 'w+', encoding=encoding) as _f:
        json5.dump(obj, _f, ensure_ascii=ensure_ascii, indent=indent)

def load_json5(path: str):
    with open(path, 'rb') as _f:
        return json5.load(_f)

def to_json(obj, path: str, encoding='utf8', ensure_ascii=False, indent=None, auto_mkdirs=True):
    """ Serialize obj as a JSON formatted stream to ```path```` """
    if auto_mkdirs:
        checkout_file_path(path)
    with open(path, 'w+', encoding=encoding) as _f:
        json.dump(obj, _f, ensure_ascii=ensure_ascii, indent=indent)

def load_json(path: str):
    """ load json file from ```path``` """
    with open(path, 'rb') as _f:
        return json.load(_f)

def to_file(path, data, mode='w+', encoding=..., auto_mkdirs=True):
    """write file and auto create dir not existed """
    if auto_mkdirs:
        checkout_file_path(path)
    if 'b' in mode:
        if encoding is ...:
            encoding = None
    elif encoding is ...:
        encoding = 'utf8'
    with open(path, mode, encoding=encoding) as _f:
        return _f.write(data)

def load_file(path, encoding=None):
    '''read file auto-closed'''
    with open(path, 'rb') as _f:
        content = _f.read()
    if encoding is None:
        return content
    return content.decode(encoding=encoding)

def decode(_bytes: bytes):
    """decide encoding and decode `_bytes`"""
    assert isinstance(_bytes, bytes)
    for encoding in ["utf8", 'gbk', 'utf16', 'utf32']:
        try:
            return _bytes.decode(encoding)
        except: pass
    raise ValueError("bytes must be {utf8, gbk, utf16, utf32}")

def encode(string: str):
    assert isinstance(string, str)
    for enc in ["utf8", 'gbk', 'utf16', 'utf32']:
        try:
            return string.encode(enc)
        except: pass
    raise ValueError("bytes must be {utf8, gbk, utf16, utf32}")


def cat(*path):
    file_bytes = bytes()
    for p in path:
        if os.path.isfile(p):
            file_bytes += load_file(p)
        else:
            raise ValueError(f"{p} is not a file")
    return file_bytes

if __name__ == '__main__':
    data = {
        "test": 111
    }
    to_json(data, "./test/output.json")
    to_json5(data, "./test/output.json5")


from threading import Thread
from collections.abc import Callable, Iterable, Mapping
from typing import Any

class ControlledThread(Thread):
    def __init__(self, group: None = None, target: Callable[..., object] | None = None, name: str | None = None, args: Iterable[Any] = (), kwargs: Mapping[str, Any] | None = None, *, daemon: bool | None = None) -> None:
        super().__init__(group, target, name, args, kwargs, daemon=daemon)
        self._status = True
        self._target = target
        self._args = args
        self._kwargs = {} if kwargs is None else kwargs

    def before_run(self): pass
    
    def after_run(self): pass
    
    def step(self):
        raise Exception("ControlledThread.step must implement.")
    
    @property
    def status(self):
        return self._status
    
    def run(self):
        self._status = True
        if self._target is not None:
            try:
                while self._status:
                    self._target(*self._args, **self._kwargs)
            finally:
                del self._target, self._args, self._kwargs
        else:
            self.before_run()
            while self._status:
                self.step()
            self.after_run()
        
    def stop(self):
        self._status = False

if __name__ == '__main__':
    def p():
        print("hi")
    class __Test(ControlledThread):
        def step(self):
            p()
            
    # t = __Test()
    t = ControlledThread(target=p)
    t.start()
    
    import time
    time.sleep(3)
    t.stop()


import multiprocessing
import gc
import threading
import os
from tqdm.asyncio import tqdm
import itertools
import asyncio
import time
from .utils import *



class Cracker:
    
    def __init__(self, path: str) -> None:
        assert os.path.exists(path)
        self.path = path

    def in_multiprocess_by_seed(self, seed=PWD_SEED_COMPLEX, 
            min_length=None, max_length=None, n_processes=None, slice_size=500,
            gc_interval=2, progressbar=False):
        pool = multiprocessing.Pool(n_processes)
        it = key_generator(seed, min_length, max_length)
        manager = multiprocessing.Manager()
        info = manager.dict()
        info['key'] = None
        if progressbar:
            bar = tqdm(mininterval=0.25)
        def cb(re):
            if progressbar:
                bar.update(slice_size)
            if re:
                info['key'] = re
                pool.terminate()
        def create_gc_timer():
            gc_timer = threading.Timer(gc_interval, create_gc_timer)
            gc_timer.start()
            gc.collect()
        create_gc_timer()
        # gc_cnt = 1
        while True:
            try:
                pwds_slice = itertools.islice(it, 0, slice_size)
                if pwds_list := list(pwds_slice):
                    pwds, char_sizes = list(zip(*pwds_list))
                    char_size = char_sizes[-1]
                    pool.apply_async(self.step_some, (*pwds, ), callback=cb, error_callback=print)
                    del pwds
                    del pwds_list
                    del char_sizes
                else:
                    pool.close()
                    pool.join()
                    pool.terminate()
                    return info['key']
                del pwds_slice
            except Exception as e:
                if e.args[0] == 'Pool not running':
                    return info['key']
                else:
                    raise e
            if progressbar:
                bar.set_postfix({
                    "char": char_size
                }, refresh=False)
            # gc.collect()

    def in_multiprocess_by_book(self, book:list, n_processes=None, chunksize=1000):
        pool = multiprocessing.Pool(n_processes)
        manager = multiprocessing.Manager()
        info = manager.dict()
        info['key'] = None
        def cb(pwds):
            pwds = filter(None, pwds)
            for pwd in pwds:
                info['key'] = pwd
                pool.terminate()
                return
        pool.map_async(self.step_once, book, chunksize=chunksize, callback=cb, error_callback=print)
        pool.close()
        pool.join()
        pool.terminate()
        return info['key']
    
    def step_once(self, pwd: bytes) -> None | bytes: pass
    def step_some(self, *pwd: bytes) -> None | bytes: pass
        
    def only_number(self, start=0, end=None, step=1, progressbar=False):
        start = int(start)
        if end is None:
            now = start
            while True:
                pwd = str(now).encode("utf8")
                if self.step_once(pwd):
                    return pwd
                now += step
        end = int(end)
        bar = range(start, end, step)
        if progressbar:
            bar = tqdm(bar)
        for number in bar:
            pwd = str(number).encode("utf8")
            if self.step_once(pwd):
                return pwd
        return None
    
    async def by_seed_async(self, seed=PWD_SEED_COMPLEX, min_length=None, max_length=None, progressbar=False):
        async def block(pwd):
            if self.step_once(pwd):
                return pwd
            return False
        if progressbar: 
            with tqdm(key_generator(seed, min_length, max_length)) as bar:
                async for pwd, _ in bar:
                    if await block(pwd):
                        return pwd
        else:
            for pwd, _ in key_generator(seed, min_length, max_length):
                if await block(pwd):
                    return pwd
        return None

    def by_seed(self, seed=PWD_SEED_COMPLEX, min_length=None, max_length=None, progressbar=False):
        loop = asyncio.get_event_loop()
        future = asyncio.ensure_future(self.by_seed_async(seed=seed, min_length=min_length, max_length=max_length, progressbar=progressbar))
        loop.run_until_complete(future)
        pwd = future.result()
        return pwd

    async def by_book_async(self, book: list[bytes], progressbar=False):
        async def block(pwd):
            if self.step_once(pwd):
                return pwd
            return False
        if progressbar:
            with tqdm(book) as bar:
                async for pwd in bar:
                    if await block(pwd):
                        return pwd
        else:
            for pwd in book:
                if await block(pwd):
                    return pwd
        return None

    def by_book(self, book: list[bytes], progressbar=False):
        loop = asyncio.get_event_loop()
        future = asyncio.ensure_future(self.by_book_async(book, progressbar=progressbar))
        loop.run_until_complete(future)
        pwd = future.result()
        return pwd


from .zip import ZipCracker
from .pdf import PdfCracker
from . import utils
class Crack:
    @staticmethod
    def zip(path: str):
        return ZipCracker(path)
    @staticmethod
    def pdf(path: str):
        return PdfCracker(path)



from test_imp import meo
import os

zip_path = os.path.join(meo.utils.script_path(__file__), "../examples/data/flag.zip")

def test_zip_crack():
    zc = meo.crack.Crack.zip(zip_path)
    real_pwd = b'a4c'
    assert zc.by_seed(meo.crack.utils.PWD_SEED_SIMPLE) == real_pwd
    assert zc.in_multiprocess_by_seed(meo.crack.utils.PWD_SEED_SIMPLE) == real_pwd


#!/usr/bin/env python
import os
import shutil
from setuptools import setup, find_packages

CUR_PATH = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(CUR_PATH, "build")
if os.path.isdir(path):
    print("INFO DEL DIR ", path)
    shutil.rmtree(path)
path = os.path.join(CUR_PATH, "dist")
if os.path.isdir(path):
    print("INFO DEL DIR ", path)
    shutil.rmtree(path)

info = {}
with open(os.path.join(CUR_PATH, "meo/__version__.py"), 'r+', encoding='utf8') as f:
    exec(f.read(), info)

with open(os.path.join(CUR_PATH, "description.md"), 'r+', encoding='utf8') as f:
    long_description = f.read()

setup(
    name         =  info['__title__'],
    author       =  info["__author__"],
    url          =  info["__url__"],
    description  =  info["__description__"],
    version      =  '.'.join(map(str, info["__version__"])),
    license      =  info["__license__"],
    author_email =  info["__author_email__"],
    packages     = find_packages(),
    include_package_data = True,
    long_description     = long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'json5>=0.9',
        "packaging",
        'pyzipper>=0.3.6',
    ],
)



"""
package declaration
"""
__title__ = 'meo'
__description__ = 'frequently-used function library'
__version__ = [0, 1, 12]
__url__ = "http://github.com/miaobuao/meo"
__author__ = 'miaobuao'
__author_email__ = 'miaobuao@outlook.com'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2022 miaobuao'



from itertools import permutations

PWD_SEED_NUMBER = b"0123456789"
PWD_SEED_LOWER_ALPHABET = b'qwertyuiopasdfghjklzxcvbnm'
PWD_SEED_UPPER_ALPHABET = b'QWERTYUIOPASDFGHJKLZXCVBNM'
PWD_SEED_SPECIAL_CHAR = b'!@#$%^&*()_+-=[{}]\|;:,<.>/?\'" '
PWD_SEED_ALPHABET = PWD_SEED_LOWER_ALPHABET + PWD_SEED_UPPER_ALPHABET
PWD_SEED_SIMPLE = PWD_SEED_LOWER_ALPHABET + PWD_SEED_NUMBER
PWD_SEED_COMPLEX = PWD_SEED_ALPHABET + PWD_SEED_SPECIAL_CHAR + PWD_SEED_NUMBER

           
def key_permution(seed, size):
    for k in permutations(seed, size):
        yield bytes(k)

def key_generator(seed=PWD_SEED_COMPLEX, start=None, end=None):
    n_size = 1 if start is None else max(int(start), 1)
    if end is None:
        while True:
            for k in key_permution(seed, n_size):
                yield k, n_size
            n_size += 1
    else:
        end = int(end)
        while True:
            if n_size <= end:
                for k in key_permution(seed, n_size):
                    yield k, n_size
            else:
                return StopIteration()
            n_size += 1
